#/bin/sh
#
# LI18NUX2K.L1/annex/locale/locale.sh
#
# Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
# (X/Open)
# trading as The Open Group
# Copyright (c) 2001, 2003 International Business Machines Corp.
# All rights reserved except as granted by this License.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the "Artistic License" which comes with this
# Kit, with the following modification:
# a) "executable(s)" should be interpreted to include
# "test case(s)"
# b) if you wish to make changes as defined in clause 2 and 3, and
# distribute a modified version of this package, then
# clauses 3c and 4c are required
# c) Clause 7 is rephrased as follows: "Subroutines supplied by you
# and linked into this Package shall not be considered part of this
# Package".
#
#
# X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
# LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# You should have received a copy of the Artistic License with this
# Kit, in the file named "Artistic".  If not, we'll be glad to provide one.

# t1.sh : test case for checking the requirements 
#              to the locale described in Annex-B are satisfied.

tet_startup=""			# startup function
tet_cleanup=""			# cleanup function
iclist="ic1"
ic1="tp1 tp2 tp3 tp4"

CheckLocale()
{
	locale=$1
	locale -a | egrep "^$locale$" > /dev/null 2>&1
	if [ $? = 0 ] ; then
		return 0
	else
		return 1
	fi
}

Output()
{
	instring=$1
	itemsperline=$2		# number of items per line
	message=$3
	count=0
	num=`expr $itemsperline`
	outstring=""

	tet_infoline "$message"
	tet_infoline " "
	for item in $instring
	do
		outstring="$outstring $item"
		count=`expr $count + 1`
		mod=`expr $count % $num`
		if [ $mod = 0 ] ; then
			tet_infoline "$outstring"
			outstring=""
		fi
	done
	if [ "$outstring" != "" ] ; then
		tet_infoline "$outstring"
	fi
}

tp1()
{
	tet_infoline "* Check that C and POSIX locales are provided."
	tet_infoline " "

	if CheckLocale C && CheckLocale POSIX ; then
		tet_result PASS  # success
	else
		tet_result FAIL  # means fail
	fi
}

tp2()
{
	tet_infoline "* Check that some locales listed in Annex B (except the locale which marked with [Support of this locale is level2]) are provided and the others are documented how to install it."
	tet_infoline " "

	unsupported=0
	locstr=""
	for loc in `cat LI18NUX2KL1_AnnexB_locales`
	do
		locale -a | egrep "^${loc}" > /dev/null 2>&1
		if [ $? != 0 ] ; then
			locstr="$locstr $loc"
		fi
	done

	if [ "$locstr" = "" ] ; then
		tet_result PASS
	else
		echo ${locstr} > hoge
		Output "$locstr" 8 "The following locales are not installed."
		tet_result FIP
	fi
}

tp3()
{
	tet_infoline "* Check that the above locales are supporting UTF-8 as a codeset at least."
	tet_infoline " "

	supstr=""
	unsupstr=""
	for lang in `cat LI18NUX2KL1_AnnexB_locales`
	do
		supported=0
		for curlang in `locale -a | fgrep $lang`
		do
			codeset=`LC_ALL=$curlang locale -k charmap | \
			  awk -F"=" '{print $2}' | sed 's/\"//g' | tr A-Z a-z`
			if [ "$codeset" = "utf-8" ] || \
				[ "$codeset" = "utf8" ] ; then
				supported=1
				break
			fi
		done
		if [ $supported = 1 ] ; then
			supstr="$supstr $lang"
		else
			unsupstr="$unsupstr $lang"
		fi
	done

	if [ "$unsupstr" = "" ] ; then
#		Output "$supstr" 8 "*** Locales which are supporting UTF8"
		tet_result PASS
	else
#		Output "$supstr" 8 "*** Locales which are supporting UTF8"
		Output "$unsupstr" 8 "Unsupported UTF8 Locales in this system:"
		tet_result FIP
	fi
}

tp4()
{
	tet_infoline "* If a codeset except for UTF-8 is supported on a locale, check that the converter object between the codeset and UTF-8 is provided."
	tet_infoline " "

	unsupstr=""
	for lang in `cat LI18NUX2KL1_AnnexB_locales`
	do
		for curlang in `locale -a | fgrep $lang`
		do
			codeset=`LC_ALL=$curlang locale -k charmap | \
			  awk -F"=" '{print $2}' | sed 's/\"//g' | tr A-Z a-z`
			if [ "$codeset" = "utf-8" ] || \
				[ "$codeset" = "utf8" ] ; then
				break
			else
				echo $unsupstr | fgrep $codeset > /dev/null 2>&1
				if [ $? != 0 ] ; then
					unsupstr="$unsupstr $codeset"
				fi
			fi
		done
	done

	if [ "$unsupstr" = "" ] ; then
		tet_result PASS
	else
		supcs=""
		unsupcs=""
		for cs in $unsupstr
		do
			iconv -l | fgrep -i $cs > /dev/null 2>&1
			if [ $? = 0 ] ; then
				supcs="$supcs ${cs}"
			else
				unsupcs="$unsupcs ${cs}"
			fi
		done
		if [ "$unsupcs" = "" ] ; then
#			Output "$unsupstr" 6 \
#"*** Some locales which support codesets except for UTF-8 were found.
#Following is the codesets list which those locales are supporting."
			tet_result PASS
		else
			Output "$unsupcs" 6 \
"No converter objects to UTF8:"
			tet_result UNSUPPORTED
		fi
	fi
}

# execute shell test case manager - must be last line
. $TET_ROOT/lib/xpg3sh/tcm.sh
