/*
 * LI18NUX2K.L1/base/__wcstoul_internal/__wcstoul_internal.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wctype.h>
#include <errno.h>

#include <stdint.h>
#include <limits.h>
#define __USE_ISOC99 1
#include <wchar.h>
#include <stddef.h>
#include <inttypes.h>

#define STR_LENGTH 32
#define INTERNAL_FLAG 0

private char *test_locale = "LTP_1.UTF-8";
private char *c_locale = "C";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();
private void test11();
private void test12();
private void test13();
private void test14();
private void test15();

#define NO_TESTS 15
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
      {test1,   1},
      {test2,   2},
      {test3,   3},
      {test4,   4},
      {test5,   5},
      {test6,   6},
      {test7,   7},
      {test8,   8},
      {test9,   9},
      {test10, 10},
      {test11, 11},
      {test12, 12},
      {test13, 13},
      {test14, 14},
      {test15, 15},
      {NULL, 0}
};


private void
test1()
{
  unsigned long int ret, exp = 4UL;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = {0x0009, 0x000A, 0x3000, 0xFF3F, L'4', L'\0'};

  DBUG_ENTER ("test1");
  tet_infoline ("Verify that this function ignores first white-space");
  tet_infoline ("characters at nptr. The white-space characters are the");
  tet_infoline ("ones whose results applied with iswspace() is true.");

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG)) == exp)
    PATH_TRACE;
  else
    {
      char msg[256];
      tet_infoline ("__wcstoul_internal() does not skip the wide space character.");
      sprintf (msg, "Return Value: %lu", ret);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test2()
{
  unsigned long int ret, exp = 7UL * 8UL + 7UL;
  wchar_t str[] = L"077";
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test2");
  tet_infoline ("When base is 0 and the wide character string for conversion");
  tet_infoline ("starts with '0', verify that the string is interpreted as");
  tet_infoline ("octal number.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("The return value is not correct.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test3()
{
  unsigned long int ret, exp = 77UL;
  wchar_t str[] = L"77";
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test3");
  tet_infoline ("When base is 0 and the wide character string for conversion");
  tet_infoline ("starts without '0', verify that the string is interpreted");
  tet_infoline ("as decimal number.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("The return value is not correct.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test4()
{
  unsigned long int ret, exp = 7UL * 16UL + 7UL;
  wchar_t str[] = L"0x77";
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test4");
  tet_infoline ("When base is 0 and the wide character string for conversion");
  tet_infoline ("starts with '0x' or '0X', verify that the string is");
  tet_infoline ("interpreted as hexadecimal number.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("The return value is not correct.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test5()
{
  unsigned long int ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"4UL", left_wc = L'U';
  wchar_t *endptr;

  DBUG_ENTER ("test5");
  tet_infoline ("When the the wide character string for conversion have a");
  tet_infoline ("integer suffix, verify that this function does not");
  tet_infoline ("interpret it as numerical symbol.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, &endptr, 0, INTERNAL_FLAG);
  DBUG_PRINTF ("test5", "endptr: %lc", *endptr);
  
  if (*endptr == left_wc)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("This function does not ignore a floating suffix.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test6()
{
  unsigned long int ret, exp = 35UL * 36UL + 35UL;
  wchar_t str[] = L"zz";
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test6");
  tet_infoline ("When base is bigger than 9, verify that some alphabets are");
  tet_infoline ("interpreted as numeric character.");
  tet_infoline ("(e.g. If base is 16, 'a', 'b', 'c', 'd', 'e', 'f' are");
  tet_infoline ("interpreted respectively '10', '11', '12', '13', '14', '15')");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 36, INTERNAL_FLAG);
  
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("This function does not recognized the base value.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test7()
{
  unsigned long int ret, exp = 35UL * 36UL + 35UL;
  wchar_t str[] = L"zZ";
  char message[64] = "Test Locale is ";
  int  pathok = 0;

  DBUG_ENTER ("test7");
  tet_infoline ("When alphabets are interpreted as numeric characters,");
  tet_infoline ("verify that both the uppercase letter and lowercase letter");
  tet_infoline ("are acceptable.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 36, INTERNAL_FLAG);
  
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("This function does not ignore the case of alphabet.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test8()
{
  unsigned long int ret;
  int  pathok = 0;
  wchar_t str[] = L"46", left_wc = L'6';
  int base = 5;
  wchar_t *endptr;
  char message[64] = "Test Locale is ";

  DBUG_ENTER ("test8");
  tet_infoline ("When a digit is bigger than base, verify that it is the");
  tet_infoline ("outside for conversion.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, &endptr, base, INTERNAL_FLAG);
  
  if (*endptr == left_wc)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      tet_infoline ("The digit which is bigger than base is not ignore.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test9()
{
  unsigned long int ret, exp = 15UL * 16UL + 15UL;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str1[] = L"0xff", str2[] = L"0XFF";
  wchar_t *endptr;

  DBUG_ENTER ("test9");
  tet_infoline ("When base is 16, verify that the wide character string for");
  tet_infoline ("conversion is permitted to have '0x' or '0X'.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str1, NULL, 16, INTERNAL_FLAG);
  
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      in_rpt ("This function does not ignore \"0x\"");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  ret = __wcstoul_internal (str2, &endptr, 16, INTERNAL_FLAG);
  
  if (ret == exp)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Return Value: %lu", ret);
      in_rpt ("This function does not ignore \"0X\"");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


private void
test10()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"4A";
  wchar_t exp = L'A', *ptr;

  DBUG_ENTER ("test10");
  tet_infoline ("When endptr is not a null pointer, verify that a pointer");
  tet_infoline ("to the character which is next to the last converted");
  tet_infoline ("character is stored in the object pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  __wcstoul_internal (str, &ptr, 0, INTERNAL_FLAG);

  if (ptr != NULL && *ptr == exp)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", *ptr, exp);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test11()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"";
  unsigned long int ret;

  DBUG_ENTER ("test11");
  tet_infoline ("When the wide character string for conversion is empty,");
  tet_infoline ("verify that return value is 0.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);

  if (ret == 0UL)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "ret: %lu, correct: %lu", ret, 0UL);
      tet_infoline (msg);
      tet_infoline ("The return value is not correct.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test12()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"";
  wchar_t *ptr;

  DBUG_ENTER ("test12");
  tet_infoline ("When the wide character string for conversion is empty and");
  tet_infoline ("endptr is not a null pointer, verify that the value of nptr");
  tet_infoline ("is stored in the object pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  __wcstoul_internal (str, &ptr, 0, INTERNAL_FLAG);

  if (ptr != NULL && ptr == str)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", ptr, str);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test13()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"Z 4";
  unsigned long int ret;

  DBUG_ENTER ("test13");
  tet_infoline ("When the wide character string for conversion does not");
  tet_infoline ("have the expected form, verify that return value is 0.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);

  if (ret == 0UL)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "ret: %lu, correct: %lu", ret, 0UL);
      tet_infoline (msg);
      tet_infoline ("The return value is not correct.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


private void
test14()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  wchar_t str[] = L"z 4";
  wchar_t *ptr;

  DBUG_ENTER ("test14");
  tet_infoline ("When the wide character string for conversion does not");
  tet_infoline ("have the expected form and endptr is not a null pointer,");
  tet_infoline ("verify that the value of nptr is stored in the object");
  tet_infoline ("pointed to by endptr.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;
  __wcstoul_internal (str, &ptr, 0, INTERNAL_FLAG);

  if (ptr != NULL && ptr == str)
    PATH_TRACE;
  else
    {
      char msg[256];
      sprintf (msg, "endptr: %X, exp: %X", ptr, str);
      tet_infoline (msg);
      tet_infoline ("ptr doesn't pointed correct place.");
      xx_rpt (FAILURE);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}

private void
test15()
{
  unsigned long int ret, max;
  wchar_t *str;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  size_t i, figure;

  DBUG_ENTER ("test15");
  tet_infoline ("When the wide character string for conversion is bigger");
  tet_infoline ("than LLONG_MAX, verify that the return value is ULONG_MAX");
  tet_infoline ("and errno is ERANGE.");

  setlocale(LC_ALL, c_locale);
  strcat (message, c_locale);
  tet_infoline (message);

  globok = 1;

  max = ULONG_MAX;

  figure = 0;
  while (max > 0)
    {
      max = max / 10;
      ++figure;
    }

  str = calloc (figure + 2, sizeof(wchar_t));
  for (i = 0; i < figure + 1; i++)
    str[i] = L'9';

  ret = __wcstoul_internal (str, NULL, 0, INTERNAL_FLAG);
  if (ret == ULONG_MAX)
    PATH_TRACE;
  else
    {
      char msg[256];

      sprintf (msg, "Str: %ls Return Value: %lu", str, ret);
      tet_infoline ("The return value is not correct.");
      tet_infoline (msg);
      xx_rpt (FAILURE);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }

  free (str);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}
