/*
 * LI18NUX2K.L1/base/fputws/fputws.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <errno.h>
#include <locale.h>
#include <wchar.h>
#include <limits.h>

private char *test_locale = "LTP_1.UTF-8";

#define	CHAR_NUM  3
#define STR_NUM   2
private wchar_t wstr[][CHAR_NUM + 2] = {{L'A', 0x00C4, 0x3042, L' ', L'\0'},
				        {0x00C4, 0x3042, L'A', L' ', L'\0'}};
private char *str[STR_NUM] = {"AÄあ", "ÄあA"};

private void test1();
private void test2();
private void test3();

#define NO_TESTS 3
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	NULL, 0
};


/* If the wide character string starting at s has possibility to convert
   multibyte character string on the current locale, verify this function
   writes the multibyte character string (except terminating null wide
   character) to the stream and returns 0 or positive value. */
private void
test1()
{
  int i;
  int ret;
  char mbs[MB_LEN_MAX * CHAR_NUM + 2];
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string starting at s has possibility "
	       "to convert multibyte character string on the current locale, "
	       "verify this function writes the multibyte character string "
	       "(except terminating null wide character) to the stream and "
	       "returns 0 or positive value.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  for (i = 0; i < STR_NUM; i++)
    {
      if ((fp = fopen ("testfile", "w")) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Cannot open 'testfile'");
	  DBUG_VOID_RETURN;
	}

      ret = fputws (wstr[i], fp);
      if (ret < 0)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The return value is not correct.");
	  DBUG_PRINTF ("test1", "Return Value: %d", ret);
	  fclose (fp);
	  continue;
	}

      fclose (fp);

      if ((fp = fopen ("testfile", "r")) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Cannot open 'testfile'");
	  DBUG_VOID_RETURN;
	}

      fscanf (fp, "%s", mbs);
      if (strcmp (mbs, str[i]) == 0)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The put string is not correct.");
	  DBUG_PRINTF ("test1", "The put string: %s", mbs);
	}

      fclose (fp);
    }

  PATH_XS_RPT (STR_NUM);
  DBUG_VOID_RETURN;
}


/* If the wide character string starting at s cannot be outputted for some
   reason, verify this function returns -1. */
private void
test2()
{
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string starting at s cannot be outputted "
	       "for some reason, verify this function returns -1.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      PATH_TRACE;
      xx_rpt (FAILURE);
      in_rpt ("Cannot open testfile for some reason.");
    }

  ret = fputws (wstr[0], fp);
  if (ret == -1)
    PATH_TRACE;
  else
    {
      PATH_TRACE;
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  fclose (fp);

  PATH_XS_RPT (1);
  DBUG_VOID_RETURN;
}


/* After executing this function correctly, verify the type of stream is
   changed to wide character oriented. */
private void
test3()
{
  wint_t  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "After executing this function correctly, verify the type of "
	       "stream is changed to wide character oriented.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  ret = fputws (wstr[0], fp);

  if (fwide (fp, 0) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
