/*
 * LI18NUX2K.L1/base/fwide/fwide.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <locale.h>
#include <wchar.h>

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();

#define NO_TESTS 4
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	test4,  4,
	NULL, 0
};


/* If any I/O function does not execute after fopen opens a file (or freopen
   reopens a file) and mode is 0, verify this function returns 0 and does not
   change the stream type. */
private void
test1()
{
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If any I/O function does not execute after fopen opens a file "
	       "(or freopen reopens a file) and mode is 0, verify this "
	       "function returns 0 and does not change the stream type.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  if ((fp = fopen ("testfile", "w")) == NULL)
      {
	xx_rpt (FAILURE);
	in_rpt ("Cannot open 'testfile'");
	DBUG_VOID_RETURN;
      }

  globok = 1;

  if (ret = fwide (fp, 0) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test1", "Return Value: %d", ret);
    }

  if (ret = fwide (fp, 0) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
      DBUG_PRINTF ("test1", "Return Value: %d", ret);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If any I/O function does not execute after fopen opens a file (or freopen
   reopens a file) and mode is positive, verify this function returns
   positive and changes the stream type to wide character oriented stream. */
private void
test2()
{
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If any I/O function does not execute after fopen opens a file "
	       "(or freopen reopens a file) and mode is positive, verify "
	       "this function returns positive and changes the stream type "
	       "to wide character oriented stream.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  if ((fp = fopen ("testfile", "w")) == NULL)
      {
	xx_rpt (FAILURE);
	in_rpt ("Cannot open 'testfile'");
	DBUG_VOID_RETURN;
      }

  globok = 1;

  if (ret = fwide (fp, 1) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test2", "Return Value: %d", ret);
    }

  if (ret = fwide (fp, 0) > 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
      DBUG_PRINTF ("test2", "Return Value: %d", ret);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If any I/O function does not execute after fopen opens a file (or freopen
   reopens a file) and mode is negative, verify this function returns negative
   and changes the stream type to byte-oriented stream. */
private void
test3()
{
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If any I/O function does not execute after fopen opens a "
	       "file (or freopen reopens a file) and mode is negative, verify "
	       "this function returns negative and changes the stream type to "
	       "byte-oriented stream.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  if ((fp = fopen ("testfile", "w")) == NULL)
      {
	xx_rpt (FAILURE);
	in_rpt ("Cannot open 'testfile'");
	DBUG_VOID_RETURN;
      }

  globok = 1;

  if (ret = fwide (fp, -1) < 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test3", "Return Value: %d", ret);
    }

  if (ret = fwide (fp, 0) < 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
      DBUG_PRINTF ("test3", "Return Value: %d", ret);
    }

  fclose (fp);

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the stream type is already defined, verify this function does not change
   the stream type whatever mode is given and returns negative or positive
   value depending on the stream type. */
private void
test4()
{
  int  ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the stream type is already defined, verify this function "
	       "does not change the stream type whatever mode is given and "
	       "returns negative or positive value depending on the stream "
	       "type.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  if ((fp = fopen ("testfile", "w")) == NULL)
      {
	xx_rpt (FAILURE);
	in_rpt ("Cannot open 'testfile'");
	DBUG_VOID_RETURN;
      }

  globok = 1;

  fputs ("This is a test file.", fp);

  if (ret = fwide (fp, 1) < 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The stream type is changed by a mistake.");
      DBUG_PRINTF ("test4", "Return Value: %d", ret);
    }

  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}
