/*
 * LI18NUX2K.L1/base/setlocale/setlocale.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <locale.h>
#include <wchar.h>

/* For testing the setlocale is called locale database correctly. */
#include <stdio.h>
#include <string.h>
#include <langinfo.h>
#include <wctype.h>
#define IS_LC_COLLATE_LPT1	(strcoll ("Cz", "Ch") < 0)
#define IS_LC_CTYPE_LPT1	(iswxdigit(L'z') == 1)
#define IS_LC_TIME_LPT1		(strcmp (nl_langinfo(MON_1), "睦a") == 0)
#define IS_LC_MONETARY_LPT1	(strcmp (nl_langinfo(CRNCYSTR), "-♪") == 0)
#define IS_LC_NUMERIC_LPT1	(strcmp (nl_langinfo(RADIXCHAR), "*") == 0)
#define IS_LC_MESSAGES_LPT1	(strcmp (nl_langinfo(YESEXPR), \
						"^([■□☆○]|♭┼|◎╂)") == 0)

private void test1();
private void test2();
private void test3();
private void test4();

#define NO_TESTS	4
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)() = NULL;
public 	void  	(*clnfunc)() = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	NULL, 0
};


/* If locale is "", verify each category of the locale is set according to
   the internationalization environment variables. */
private void
test1()
{ 
  int pathok = 0;
  char *desc = "If locale is \"\", verify each category of the locale is set "
	       "according to the internationalization environment variables.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  tet_infoline ("* Notice *");
  tet_infoline ("This test uses strcoll(), isxdigit(), strcmp().");
  tet_infoline ("Please also doubt these functions when this test is failed.");

  globok = 1;
  
  /* unset LC_ALL, LC_COLLATE, LC_CTYPE, LC_MESSAGES, LC_MONETARY,
	   LC_NUMERIC, LC_TIME
     export LANG=LTP_1.UTF-8 */
  unsetenv ("LC_ALL");	unsetenv ("LC_COLLATE");  unsetenv ("LC_CTYPE");
  unsetenv ("LC_MESSAGES"); unsetenv ("LC_MONETARY"); unsetenv ("LC_NUMERIC");
  unsetenv ("LC_TIME");
  setenv ("LANG", "LTP_1.UTF-8", 1);

  /* Set each category according to environment variable. */
  setlocale (LC_COLLATE, "");
  setlocale (LC_CTYPE, "");
  setlocale (LC_TIME, "");
  setlocale (LC_MONETARY, "");
  setlocale (LC_NUMERIC, "");
  setlocale (LC_MESSAGES, "");

  if (IS_LC_COLLATE_LPT1 && IS_LC_CTYPE_LPT1 && IS_LC_TIME_LPT1 &&
      IS_LC_MONETARY_LPT1 && IS_LC_NUMERIC_LPT1 && IS_LC_MESSAGES_LPT1)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test1", "LANG -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Some category is not set correctry");
    }

  /* export LC_COLLATE=C; export LC_CTYPE=C; export LC_MESSAGES=C;
     export LC_MONETARY=C; export LC_NUMERIC=C; export LC_TIME=C; */
  setenv ("LC_COLLATE",  "C", 1);
  setenv ("LC_CTYPE",    "C", 1);
  setenv ("LC_MESSAGES", "C", 1);
  setenv ("LC_MONETARY", "C", 1);
  setenv ("LC_NUMERIC",  "C", 1);
  setenv ("LC_TIME",     "C", 1);

  /* Set each category according to environment variable. */
  setlocale (LC_COLLATE, "");
  setlocale (LC_CTYPE, "");
  setlocale (LC_TIME, "");
  setlocale (LC_MONETARY, "");
  setlocale (LC_NUMERIC, "");
  setlocale (LC_MESSAGES, "");

  if (!IS_LC_COLLATE_LPT1 && !IS_LC_CTYPE_LPT1 && !IS_LC_TIME_LPT1 &&
      !IS_LC_MONETARY_LPT1 && !IS_LC_NUMERIC_LPT1 && !IS_LC_MESSAGES_LPT1)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test1", "LC_* -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      DBUG_PRINTF ("test1", setlocale (LC_COLLATE, ""));
      DBUG_PRINTF ("test1", setlocale (LC_CTYPE, ""));
      DBUG_PRINTF ("test1", setlocale (LC_MESSAGES, ""));
      DBUG_PRINTF ("test1", setlocale (LC_MONETARY, ""));
      DBUG_PRINTF ("test1", setlocale (LC_NUMERIC, ""));
      DBUG_PRINTF ("test1", setlocale (LC_TIME, ""));
      in_rpt ("Some category are not set correctry");
    }

  /* export LC_ALL=LTP_1.UTF-8 */
  setenv ("LC_ALL",  "LTP_1.UTF-8",     1);

  /* Set each category according to environment variable. */
  setlocale (LC_COLLATE, "");
  setlocale (LC_CTYPE, "");
  setlocale (LC_TIME, "");
  setlocale (LC_MONETARY, "");
  setlocale (LC_NUMERIC, "");
  setlocale (LC_MESSAGES, "");

  if (IS_LC_COLLATE_LPT1 && IS_LC_CTYPE_LPT1 && IS_LC_TIME_LPT1 &&
      IS_LC_MONETARY_LPT1 && IS_LC_NUMERIC_LPT1 && IS_LC_MESSAGES_LPT1)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test1", "LC_ALL -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Some category are not set correctry");
    }


  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}


/* If the string starting at locale is a string which is used to define the
   locale, verify each category of the locale is set according to the
   string starting at locale and returns the string associated with the
   category for the new locale. */
private void
test2()
{ 
  int pathok = 0;
  char *desc = "If the string starting at locale is a string which is used "
	       "to define the locale, verify each category of the locale is "
	       "set according to the string starting at locale and returns "
	       "the string associated with the category for the new locale.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  tet_infoline ("* Notice *");
  tet_infoline ("This test uses strcoll(), isxdigit(), strcmp().");
  tet_infoline ("Please also doubt these functions when this test is failed.");

  globok = 1;
  
  /* unset LC_ALL, LC_COLLATE, LC_CTYPE, LC_MESSAGES, LC_MONETARY,
	   LC_NUMERIC, LC_TIME, LANG */
  unsetenv ("LC_ALL");	unsetenv ("LC_COLLATE");  unsetenv ("LC_CTYPE");
  unsetenv ("LC_MESSAGES"); unsetenv ("LC_MONETARY"); unsetenv ("LC_NUMERIC");
  unsetenv ("LC_TIME"); unsetenv ("LANG");

  setlocale (LC_COLLATE, "LTP_1.UTF-8");
  if (IS_LC_COLLATE_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_COLLATE: The return value is not correctry");
    }
  
  setlocale (LC_CTYPE, "LTP_1.UTF-8");
  if (IS_LC_CTYPE_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_CTYPE: The return value is not correctry");
    }

  setlocale (LC_TIME, "LTP_1.UTF-8");
  if (IS_LC_TIME_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correctry");
    }

  setlocale (LC_MONETARY, "LTP_1.UTF-8");
  if (IS_LC_MONETARY_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_MONETARY: The return value is not correctry");
    }

  setlocale (LC_NUMERIC, "LTP_1.UTF-8");
  if (IS_LC_NUMERIC_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_NUMERIC: The return value is not correctry");
    }

  setlocale (LC_MESSAGES, "LTP_1.UTF-8");
  if (IS_LC_MESSAGES_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_TIME: The return value is not correctry");
    }

  setlocale (LC_ALL, "C");
  if (!IS_LC_COLLATE_LPT1 && !IS_LC_CTYPE_LPT1 && !IS_LC_TIME_LPT1 &&
      !IS_LC_MONETARY_LPT1 && !IS_LC_NUMERIC_LPT1 && !IS_LC_MESSAGES_LPT1)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_ALL: The return value is not correctry");
    }

  PATH_XS_RPT(7);
  DBUG_VOID_RETURN;
}


/* If the string starting at locale is not a string which is used to define
   the locale, verify this function does not change any category of the
   locale and returns NULL. */
private void
test3()
{ 
  int pathok = 0;
  char *desc = "If the string starting at locale is not a string which is "
	       "used to define the locale, verify this function does not "
	       "change any category of the locale and returns NULL.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  globok = 1;
  
  /* unset LC_ALL, LC_COLLATE, LC_CTYPE, LC_MESSAGES, LC_MONETARY,
	   LC_NUMERIC, LC_TIME, LANG */
  unsetenv ("LC_ALL");	unsetenv ("LC_COLLATE");  unsetenv ("LC_CTYPE");
  unsetenv ("LC_MESSAGES"); unsetenv ("LC_MONETARY"); unsetenv ("LC_NUMERIC");
  unsetenv ("LC_TIME"); unsetenv ("LANG");

  if (setlocale (LC_COLLATE, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_COLLATE: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_CTYPE, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_CTYPE: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_MESSAGES, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_MONETARY, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_MONETARY: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_NUMERIC, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_NUMERIC: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_TIME, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_TIME: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  if (setlocale (LC_ALL, "hehehenohe") == NULL)
    {
      PATH_TRACE;
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("LC_ALL: The return value is not correctry");
      in_rpt ("* Notice *");
      in_rpt (" If you have hehehenohe locale this test must be failed.");
    }

  PATH_XS_RPT(7);
  DBUG_VOID_RETURN;
}


/* If locale is NULL, verify this function returns the string associated
   with the category for the locale. */
private void
test4()
{ 
  int pathok = 0;
  char *desc = "If locale is NULL, verify this function returns the string "
	       "associated with the category for the locale.";

  char collate_ltp1[BUFSIZ], ctype_ltp1[BUFSIZ], time_ltp1[BUFSIZ],
       monetary_ltp1[BUFSIZ], numeric_ltp1[BUFSIZ], messages_ltp1[BUFSIZ];
  char collate_c[BUFSIZ], ctype_c[BUFSIZ], time_c[BUFSIZ],
       monetary_c[BUFSIZ], numeric_c[BUFSIZ], messages_c[BUFSIZ];

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  strcpy (collate_ltp1,  setlocale (LC_COLLATE,  "LTP_1.UTF-8"));
  strcpy (ctype_ltp1,    setlocale (LC_CTYPE,    "LTP_1.UTF-8"));
  strcpy (time_ltp1,     setlocale (LC_TIME,     "LTP_1.UTF-8"));
  strcpy (monetary_ltp1, setlocale (LC_MONETARY, "LTP_1.UTF-8"));
  strcpy (numeric_ltp1,  setlocale (LC_NUMERIC,  "LTP_1.UTF-8"));
  strcpy (messages_ltp1, setlocale (LC_MESSAGES, "LTP_1.UTF-8"));

  strcpy (collate_c,  setlocale (LC_COLLATE,  "C"));
  strcpy (ctype_c,    setlocale (LC_CTYPE,    "C"));
  strcpy (time_c,     setlocale (LC_TIME,     "C"));
  strcpy (monetary_c, setlocale (LC_MONETARY, "C"));
  strcpy (numeric_c,  setlocale (LC_NUMERIC,  "C"));
  strcpy (messages_c, setlocale (LC_MESSAGES, "C"));

  globok = 1;
  
  /* unset LC_ALL, LC_COLLATE, LC_CTYPE, LC_MESSAGES, LC_MONETARY,
	   LC_NUMERIC, LC_TIME
     export LANG=LTP_1.UTF-8 */
  unsetenv ("LC_ALL");	unsetenv ("LC_COLLATE");  unsetenv ("LC_CTYPE");
  unsetenv ("LC_MESSAGES"); unsetenv ("LC_MONETARY"); unsetenv ("LC_NUMERIC");
  unsetenv ("LC_TIME");
  setenv ("LANG", "LTP_1.UTF-8", 1);

  setlocale (LC_ALL, "");

  if (strcmp (setlocale (LC_COLLATE, NULL),  collate_ltp1)  == 0 &&
      strcmp (setlocale (LC_CTYPE, NULL),    ctype_ltp1)    == 0 &&
      strcmp (setlocale (LC_MESSAGES, NULL), messages_ltp1) == 0 &&
      strcmp (setlocale (LC_MONETARY, NULL), monetary_ltp1) == 0 &&
      strcmp (setlocale (LC_NUMERIC, NULL),  numeric_ltp1)  == 0 &&
      strcmp (setlocale (LC_TIME, NULL),     time_ltp1)     == 0)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test4", "LANG -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      DBUG_PRINTF ("test4", setlocale (LC_COLLATE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_CTYPE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MESSAGES, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MONETARY, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_NUMERIC, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_TIME, NULL));
      in_rpt ("Some category is not set correctry");
    }

  /* export LC_COLLATE=C; export LC_CTYPE=C; export LC_MESSAGES=C;
     export LC_MONETARY=C; export LC_NUMERIC=C; export LC_TIME=C; */
  setenv ("LC_COLLATE",  "C", 1);
  setenv ("LC_CTYPE",    "C", 1);
  setenv ("LC_MESSAGES", "C", 1);
  setenv ("LC_MONETARY", "C", 1);
  setenv ("LC_NUMERIC",  "C", 1);
  setenv ("LC_TIME",     "C", 1);

  setlocale (LC_ALL, "");

  if (strcmp (setlocale (LC_COLLATE, NULL),  collate_c)  == 0 &&
      strcmp (setlocale (LC_CTYPE, NULL),    ctype_c)    == 0 &&
      strcmp (setlocale (LC_MESSAGES, NULL), messages_c) == 0 &&
      strcmp (setlocale (LC_MONETARY, NULL), monetary_c) == 0 &&
      strcmp (setlocale (LC_NUMERIC, NULL),  numeric_c)  == 0 &&
      strcmp (setlocale (LC_TIME, NULL),     time_c)     == 0)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test4", "LC_* -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      DBUG_PRINTF ("test4", setlocale (LC_COLLATE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_CTYPE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MESSAGES, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MONETARY, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_NUMERIC, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_TIME, NULL));
      in_rpt ("Some category are not set correctry");
    }

  /* export LC_ALL=LTP_1.UTF-8 */
  setenv ("LC_ALL",  "LTP_1.UTF-8",     1);
  setlocale (LC_ALL, "");

  if (strcmp (setlocale (LC_COLLATE, NULL),  collate_ltp1)  == 0 &&
      strcmp (setlocale (LC_CTYPE, NULL),    ctype_ltp1)    == 0 &&
      strcmp (setlocale (LC_MESSAGES, NULL), messages_ltp1) == 0 &&
      strcmp (setlocale (LC_MONETARY, NULL), monetary_ltp1) == 0 &&
      strcmp (setlocale (LC_NUMERIC, NULL),  numeric_ltp1)  == 0 &&
      strcmp (setlocale (LC_TIME, NULL),     time_ltp1)     == 0)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test4", "LC_ALL -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("Some category are not set correctry");
    }

  /* Do not rely environment variable. */
  (void)setlocale (LC_COLLATE,  "C");
  (void)setlocale (LC_CTYPE,    "C");
  (void)setlocale (LC_MESSAGES, "C");
  (void)setlocale (LC_MONETARY, "C");
  (void)setlocale (LC_NUMERIC,  "C");
  (void)setlocale (LC_TIME,     "C");

  if (strcmp (setlocale (LC_COLLATE, NULL),  collate_c)  == 0 &&
      strcmp (setlocale (LC_CTYPE, NULL),    ctype_c)    == 0 &&
      strcmp (setlocale (LC_MESSAGES, NULL), messages_c) == 0 &&
      strcmp (setlocale (LC_MONETARY, NULL), monetary_c) == 0 &&
      strcmp (setlocale (LC_NUMERIC, NULL),  numeric_c)  == 0 &&
      strcmp (setlocale (LC_TIME, NULL),     time_c)     == 0)
    {
      PATH_TRACE;
      DBUG_PRINTF ("test4", "Don't rely environment variable -- O.K.");
    }
  else
    {
      xx_rpt (FAILURE);
      DBUG_PRINTF ("test4", setlocale (LC_COLLATE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_CTYPE, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MESSAGES, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_MONETARY, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_NUMERIC, NULL));
      DBUG_PRINTF ("test4", setlocale (LC_TIME, NULL));
      in_rpt ("Some category are not set correctry");
    }

  PATH_XS_RPT(4);
  DBUG_VOID_RETURN;
}
