/*
 * LI18NUX2K.L1/base/towupper/towupper.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wchar.h>
#include <wctype.h>

#define UNICODE_MAX 0xFFFF

private char *test_locale = "LTP_1.UTF-8";
private wint_t touppercase[UNICODE_MAX];

/* private wint_t pair[][2] = { ... }; */
#include <./pairs>

private void test1();
private void test2();
private void read_pairs();

#define NO_TESTS 2
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	NULL, 0
};


/* If iswlower(wc) returns true and wc has uppercase wide character, verify
   this function returns the uppercase wide character. */
private void
test1()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  long ucode;
  char *desc = "If iswlower(wc) returns true and wc has uppercase wide "
	       "character, verify this function returns the uppercase wide "
	       "character.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale (LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  read_pairs();

  globok = 1;
  pathok = 1;

  for (ucode = 0; ucode <= UNICODE_MAX; ucode++)
    {
      char    utf8[7];
      wchar_t wc;
      size_t  mblength;
      mbstate_t state;

      memset (utf8, '\0', 7);
      unicode_to_utf8 (ucode, utf8);

      memset (&state, '\0', sizeof (mbstate_t));
      mblength = mbrtowc (&wc, utf8, 7, &state);

      if (touppercase[wc] != WEOF)
	if (towupper(wc) != touppercase[wc])
	  {
	    xx_rpt (FAILURE);
	    in_rpt ("The return value is not correct.");
	    /*
	    DBUG_PRINTF ("test1", "lower = %X", ucode);
	    */
	  }
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If iswlower(wc) returns false, or islower(wc) returns true and wc does
   not have corresponded uppercase wide character, verify this function
   returns wc. */
private void
test2()
{
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  long ucode;
  char *desc = "If iswlower(wc) returns false, or islower(wc) returns true "
	       "and wc does not have corresponded uppercase wide character, "
	       "verify this function returns wc.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale (LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  read_pairs();

  globok = 1;
  pathok = 1;

  for (ucode = 0; ucode <= UNICODE_MAX; ucode++)
    {
      char    utf8[7];
      wchar_t wc;
      size_t  mblength;
      mbstate_t state;

      memset (utf8, '\0', 7);
      unicode_to_utf8 (ucode, utf8);

      memset (&state, '\0', sizeof (mbstate_t));
      mblength = mbrtowc (&wc, utf8, 7, &state);

      if (touppercase[wc] == WEOF)
	if (towupper(wc) != wc)
	  {
	    xx_rpt (FAILURE);
	    in_rpt ("The return value is not correct.");
	    /*
	    DBUG_PRINTF ("test1", "lower = %X", ucode);
	    */
	  }
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* Encode unicode code point to UTF-8. */
int unicode_to_utf8(long ucode, char *utf8)
{
  if(ucode <= 0x7F)
    {
      utf8[0] = ucode;
      return 1;
    }
  else if (0x0080 < ucode && ucode < 0x07FF)
    {
      utf8[0] = ((ucode >> 6) & 0x1F) | 0xC0;
      utf8[1] = (ucode & 0x3F) | 0x80;
      return 2;
    }
  else if(0x0800 < ucode && 0xFFFF)
    {
      utf8[0] = ((ucode >> 12) & 0x0F) | 0xE0;
      utf8[1] = ((ucode >> 6) & 0x3f) | 0x80;
      utf8[2] = (ucode & 0x3F) | 0x80;
      return 3;
    }

  return 0;
}



/* Read corresponded uppercase characters. */
void read_pairs(void)
{
  int i, flag;

  memset (touppercase, WEOF, (UNICODE_MAX + 1) * sizeof (wint_t));
  for (i = 0; i < sizeof (pair) / sizeof (pair[0]); i++)
    touppercase[pair[i][0]] = pair[i][1];
}      
