/*
 * LI18NUX2K.L1/base/ungetwc/ungetwc.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <errno.h>
#include <locale.h>
#include <wchar.h>

#define	CHAR_NUM  3

private char *test_locale = "LTP_1.UTF-8";

private wint_t wchara[CHAR_NUM] = {L'A', 0x00C4, 0x3042};
private char *chara[CHAR_NUM]   = {"A",  "Ä",    "あ"};

struct unget_pair_tag {
  wint_t getone;
  wint_t ungetone;
};

struct unget_pair_tag unget_pair[] = {
  {L'A', L'A'},
  {0x00C4, 0x00C4},
  {0x3042, 0x3042},
  {L'A', L'B'},
  {0x00C4, 0x00CB},
  {0x3042, 0x3044},
  {L'A', 0x00C4},
  {L'A', 0x3042},
  {0x00C4, L'A'},
  {0x00C4, 0x3042},
  {0x3042, L'A'},
  {0x3042, 0x00C4}
};

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();

#define NO_TESTS 6
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	test4,  4,
	test5,  5,
	test6,  6,
	NULL, 0
};


/* Upon successful completion, verify this function returns the pushed-back
   character. */
private void
test1()
{
  int i;
  wint_t c, ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Upon successful completion, verify this function returns the "
	       "pushed-back character.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test1", "chara: %s", chara[i]);
    }

  fclose (fp);

  /* Read from the file. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      c = getwc (fp);
        DBUG_PRINTF("test1", "wchara[%d]: %lc", i, c);
      ret = ungetwc(c, fp);
        DBUG_PRINTF("test1", "ret_wchara[%d]: %lc", i, ret);

      if (ret == c)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The return value is not correct.");
	  DBUG_PRINTF ("test1", "Return Value: %lc", ret);
	}
      (void) getwc (fp);
    }

  fclose (fp);

  PATH_XS_RPT(CHAR_NUM);
  DBUG_VOID_RETURN;
}


/* After push back one wide character onto the stream with this function,
   verify the next function to read one or some characters from the stream
   gets the character as the first character. */
private void
test2()
{
  int i;
  wint_t c;
  size_t unget_pair_num;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "After push back one wide character onto the stream with this "
	       "function, verify the next function to read one or some "
	       "characters from the stream gets the character as the first "
	       "character.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test2", "chara: %s", chara[i]);
    }

  fclose (fp);

  unget_pair_num = sizeof(unget_pair) / sizeof(unget_pair[0]);
  for (i = 0; i < unget_pair_num; i++)
    {
      /* Read from the file. */
      if ((fp = fopen ("testfile", "r")) == NULL)
	{
	  xx_rpt (FAILURE);
	  in_rpt ("Cannot open 'testfile'");
	  DBUG_VOID_RETURN;
	}

      while (getwc(fp) == unget_pair[i].getone);
        DBUG_PRINTF("test2", "get one[%d]  : %lc", i, unget_pair[i].getone);
        DBUG_PRINTF("test2", "unget one[%d]: %lc", i, unget_pair[i].ungetone);
      (void) ungetwc(unget_pair[i].ungetone, fp);
      c = getwc (fp);
        DBUG_PRINTF("test2", "reget one[%d]: %lc", i, c);

      if (unget_pair[i].ungetone == c)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("An read function gets a wrong character.");
	  DBUG_PRINTF ("test2", "The character: %lc", c);
	}

      fclose (fp);
    }

  PATH_XS_RPT(unget_pair_num);
  DBUG_VOID_RETURN;
}


/* If c is WEOF, verify this function returns WEOF and does not influence
   the stream. */
private void
test3()
{
  int i;
  wint_t ret, c1, c2;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If c is WEOF, verify this function returns WEOF and does not "
	       "influence the stream.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test3", "chara: %s", chara[i]);
    }

  fclose (fp);

  /* Read from the file. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      c1 = getwc (fp);
      ret = ungetwc (WEOF, fp);
      (void) ungetwc(c1, fp);
      c2 = getwc (fp);
        DBUG_PRINTF("test3", "reget_wchara[%d]: %lc", i, c2);

      if (ret == WEOF)
	 if(c1 == c2)
	   PATH_TRACE;
	 else
	   {
	     xx_rpt (FAILURE);
	     in_rpt ("An read function gets a wrong character.");
	     DBUG_PRINTF ("test3", "The character: %lc", c2);
	   }
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The return value is not correct.");
	  DBUG_PRINTF ("test3", "Return value: %lc", ret);
	}
    }

  fclose (fp);

  PATH_XS_RPT(CHAR_NUM);
  DBUG_VOID_RETURN;
}


/* After ungetwc is called, check the end-of-file indicator is cleared. */
private void
test4()
{
  int i;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "After ungetwc is called, check the end-of-file indicator "
	       "is cleared.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test4", "chara: %s", chara[i]);
    }

  fclose (fp);

  /* Read from the file. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  while (feof(fp) == 0)
    (void)fgetwc (fp);

  /* Unget to the file. */
  (void) ungetwc (L'A', fp);

  if (feof(fp) == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The end-of-file indicator is not cleared.");
    }


  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* When the file-position indicator stand at the top of a file, verify WEOF
   is returned by calling this function or ungotten character is can be
   gotten after calling this function. */
private void
test5()
{
  int i;
  wint_t c, ret;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "When the file-position indicator stand at the top of a file, "
	       "verify WEOF is returned by calling this function or ungotten "
	       "character is can be gotten after calling this function.";

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;
  
  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test5", "chara: %s", chara[i]);
    }

  fclose (fp);

  /* `testfile is opened. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  /* Unget to the file. */
  ret = ungetwc (0x3042, fp);

  if (ret == WEOF)
    PATH_TRACE;
  else 
    {
      DBUG_PRINTF ("test5", "Unget character: %lc", ret);

      if ((c = getwc(fp)) == 0x3042)
	PATH_TRACE;
      else
	{
	  xx_rpt (FAILURE);
	  in_rpt ("The return value is not WEOF and the push-back character "
		  "is not gotten correctly.");
	  DBUG_PRINTF ("test5", "Reget character: %lc", c);
	}
    }
  fclose (fp);

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* Verify the value of the file-position indicator for the stream when this
   function is called successfully is decreased before the character was
   pushed-back. (But the file-position indicator is 0 before the function
   is called, the file position indicator is not indeterminate after the
   call.) */
private void
test6()
{
  int i, flag = 0;
  long fpos1, fpos2;
  wint_t ret, c;
  FILE *fp;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Verify the value of the file-position indicator for the "
	       "stream when this function is called successfully is "
	       "decreased before the character was pushed-back. (But the "
	       "file-position indicator is 0 before the function is called, "
	       "the file position indicator is not indeterminate after the "
	       "call.)";

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  /* Output to the file. */
  if ((fp = fopen ("testfile", "w")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  for (i = 0; i < CHAR_NUM; i++)
    {
      fprintf (fp, "%s", chara[i]);
      DBUG_PRINTF("test6", "chara: %s", chara[i]);
    }
  fclose (fp);


  /*
   * For a unibyte character.
   */

  /* `testfile' is opened. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  /* Get a character. */
  c = getwc(fp);
  fpos1 = ftell (fp);
  DBUG_PRINTF ("test6", "Get a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos1);

  /* Unget a character. */
  ret = ungetwc (c, fp);
  fpos2 = ftell (fp);
  DBUG_PRINTF ("test6", "Unget a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos2);

  if (fpos1 > fpos2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The file position is not decremented.");
    }
  fclose(fp);

  /*
   * For a dobule byte character.
   */

  /* `testfile' is opened. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  /* Skip a unibyte character. */
  c = getwc(fp);
  DBUG_PRINTF ("test6", " - Skip unibyte chara: %lc", c);

  /* Get a double byte character. */
  c = getwc(fp);
  fpos1 = ftell (fp);
  DBUG_PRINTF ("test6", "Get a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos1);

  /* Unget a double byte character. */
  ret = ungetwc (c, fp);
  fpos2 = ftell (fp);
  DBUG_PRINTF ("test6", "Unget a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos2);

  if (fpos1 > fpos2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The file position is not decremented.");
    }

  fclose (fp);


  /*
   * For a triple byte character.
   */

  /* `testfile' is opened. */
  if ((fp = fopen ("testfile", "r")) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot open 'testfile'");
      DBUG_VOID_RETURN;
    }

  /* Skip a unibyte character. */
  c = getwc(fp);
  DBUG_PRINTF ("test6", " - Skip unibyte chara: %lc", c);

  /* Skip a double character. */
  c = getwc(fp);
  DBUG_PRINTF ("test6", " - Skip double byte chara: %lc", c);

  /* Get a triple byte character. */
  c = getwc(fp);
  fpos1 = ftell (fp);
  DBUG_PRINTF ("test6", "Get a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos1);
  if (fpos1 == 0)
    {
      xx_rpt (UNTESTED);
      in_rpt ("The file-position indicator is not increased by getwc().");
      DBUG_VOID_RETURN;
    };

  /* Unget a triple byte character. */
  ret = ungetwc (c, fp);
  fpos2 = ftell (fp);
  DBUG_PRINTF ("test6", "Unget a %lc character.", c);
  DBUG_PRINTF ("test6", "Now file positon is %d", fpos2);

  if (fpos1 > fpos2)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The file position is not decremented.");
    }
  fclose (fp);

  PATH_XS_RPT(3);
  DBUG_VOID_RETURN;
}
