/*
 * LI18NUX2K.L1/base/wcscat/wcscat.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <wchar.h>

#define STRING_LENGTH 16
private wchar_t	s1[STRING_LENGTH];
private wchar_t s2[STRING_LENGTH];

private void test1();
private void test2();

#define NO_TESTS	2
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)() = NULL;
public 	void  	(*clnfunc)() = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
        test2, 2,
	NULL, 0
};

/* Verify this function returns s1. */
private void
test1()
{ 
  wchar_t *rval;
  int pathok = 0;
  char *desc = "Verify this function returns s1.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  s1[0] = L'0';  s1[1] = L'1';  s1[2] = L'2';  s1[3] = L'\0';
  s2[0] = L'A';  s2[1] = L'B';  s2[2] = L'C';  s2[3] = L'\0';
  
  globok = 1;
  rval = wcscat (s1, s2);

  if (rval == s1)
     PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* Verify this function concatenates two wide character strings correctly. */
private void
test2()
{
  int pathok = 0;
  char *desc = "Verify this function concatenates two wide character "
               "strings correctly.";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  s1[0] = L'0';  s1[1] = L'1';  s1[2] = L'2';  s1[3] = L'\0'; /* terminate */
  s1[4] = L'4';  s1[5] = L'5';  s1[6] = L'6';  s1[7] = L'7';
  s1[8] = L'8';  s1[9] = L'9';  s1[10] = L'a'; s1[11] = L'b';
  s1[12] = L'c'; s1[13] = L'd'; s1[14] = L'e'; s1[15] = L'\0';

  s2[0] = L'A';  s2[1] = L'B';  s2[2] = L'C';  s2[3] = L'\0';
   
  globok = 1;
  (void)wcscat (s1, s2);
  
  if (s1[0] == L'0')  PATH_TRACE;
  if (s1[1] == L'1')  PATH_TRACE;
  if (s1[2] == L'2')  PATH_TRACE;
  if (s1[3] == L'A')  PATH_TRACE;
  if (s1[4] == L'B')  PATH_TRACE;
  if (s1[5] == L'C')  PATH_TRACE;
  if (s1[6] == L'\0') PATH_TRACE;

  if (pathok != 7)
    in_rpt ("wcscat() does not concatenate correctly.");

  PATH_XS_RPT(7);
  DBUG_VOID_RETURN;
}
