/*
 * LI18NUX2K.L1/base/wcsncpy/wcsncpy.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <wchar.h>

#define STRING_LENGTH 16

private void test1();
private void test2();
private void test3();
private void test4();

#define NO_TESTS 4
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1, 1,
	test2, 2,
	test3, 3,
	test4, 4,
	NULL, 0
};

/* Verify this function returns the pointer s1. */
private void
test1()
{
  wchar_t *rval;
  wchar_t s1[STRING_LENGTH];
  wchar_t s2[] = L"ABC";
  int  pathok = 0;
  char *desc = "Verify this function returns the pointer s1.";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  globok = 1;
  rval = wcsncpy (s1, s2, 3);

  if (rval == s1)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
    }
                
  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the length of the string starting at s2 is longer or as same as n but
   it has null wide character which is in a position less than the nth,
   verify this function does not copy the wide characters which follows the
   null wide character. (includes null wide character.) */
private void
test2()
{
  wchar_t s1[STRING_LENGTH];
  wchar_t s2[STRING_LENGTH];
  int pathok = 0;
  char *desc = "If the length of the string starting at s2 is longer or as "
	       "same as n but it has null wide character which is in a "
	       "position less than the nth, verify this function does not "
	       "copy the wide characters which follows the null wide "
	       "character. (includes null wide character.)";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  s1[0] = L'0';  s1[1] = L'1';  s1[2] = L'2';  s1[3] = L'3';
  s1[4] = L'4';  s1[5] = L'5';  s1[6] = L'6';  s1[7] = L'\0';

  s2[0] = L'A';  s2[1] = L'B';  s2[2] = L'C';  s2[3] = L'\0';
  s2[4] = L'E';  s2[5] = L'F';  s2[6] = L'G';  s2[7] = L'\0';

  (void) wcsncpy (s1, s2, 6);

  globok = 1;
  if (s1[0] == L'A')  PATH_TRACE;
  if (s1[1] == L'B')  PATH_TRACE;
  if (s1[2] == L'C')  PATH_TRACE;
  if (s1[3] == L'\0') PATH_TRACE;
  if (s1[4] != L'E')  PATH_TRACE;
  if (s1[5] != L'F')  PATH_TRACE;
  
  if (pathok != 6)
    in_rpt ("This function does not interpret the null wide character.");

  PATH_XS_RPT (6);
  DBUG_VOID_RETURN;
}


/* If the length of the string starting at s2 is shorter than n, verify
   this function appends the null wide character to the copy in the array
   pointed to by s1, until n wide character in all are written. */
private void
test3()
{
  wchar_t s1[] = L"0123456789";
  wchar_t s2[] = L"ABCD";
  int pathok = 0;
  char *desc = "If the length of the string starting at s2 is shorter than "
	       "n, verify this function appends the null wide character to "
	       "the copy in the array pointed to by s1, until n wide "
	       "character in all are written.";

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  (void) wcsncpy (s1, s2, 8);

  globok = 1;
  if (s1[0] == L'A')  PATH_TRACE;
  if (s1[1] == L'B')  PATH_TRACE;
  if (s1[2] == L'C')  PATH_TRACE;
  if (s1[3] == L'D')  PATH_TRACE;
  if (s1[4] == L'\0') PATH_TRACE;
  if (s1[5] == L'\0') PATH_TRACE;
  if (s1[6] == L'\0') PATH_TRACE;
  if (s1[7] == L'\0') PATH_TRACE;
  if (s1[8] != L'8')  PATH_TRACE;
  if (s1[9] != L'9')  PATH_TRACE;

  if (pathok != 8)
    in_rpt ("This function does not output the null wide character correctly.");

  PATH_XS_RPT (8);
  DBUG_VOID_RETURN;
}


/* If the length of the string starting at s2 is longer than n, verify this
   function copies only n wide characters in the array pointed to by s1 and
   does not append null wide character at the end of the array. */
private void
test4()
{
  wchar_t s1[] = L"01234567";
  wchar_t s2[] = L"ABCDEFGH";
  int pathok = 0;
  char *desc = "If the length of the string starting at s2 is longer than n, "
	       "verify this function copies only n wide characters in the "
	       "array pointed to by s1 and does not append null wide "
	       "character at the end of the array.";

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  (void) wcsncpy (s1, s2, 4);

  globok = 1;
  if (s1[0] == L'A')  PATH_TRACE;
  if (s1[1] == L'B')  PATH_TRACE;
  if (s1[2] == L'C')  PATH_TRACE;
  if (s1[3] == L'D')  PATH_TRACE;
  if (s1[4] != L'E' && s1[4] != '\0') PATH_TRACE;
  if (s1[5] != L'F') PATH_TRACE;
  if (s1[6] != L'G') PATH_TRACE;
  if (s1[7] != L'H') PATH_TRACE;

  if (pathok != 8)
    in_rpt ("This function does not output the null wide character correctly.");

  PATH_XS_RPT (8);
  DBUG_VOID_RETURN;
}
