/*
 * LI18NUX2K.L1/base/wcstod/wcstod.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wchar.h>
#include <wctype.h>
#include <math.h>
#include <errno.h>

#define STR_LENGTH 16

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();

#define NO_TESTS 8
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,  1,
	test2,  2,
	test3,  3,
	test4,  4,
	test5,  5,
	test6,  6,
	test7,  7,
	test8,  8,
	NULL, 0
};


/* Check this function skips whitespace characters which is at beginning of
   the starting at nptr. The whitespace character is the one whose results
   applied with iswspace() is true. */
private void
test1()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Check this function skips whitespace characters which is at "
	       "beginning of the starting at nptr. The whitespace character "
	       "is the one whose results applied with iswspace() is true.";

  wchar_t str[] = L"    4";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = wcstod (str, NULL)) == 4)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("wcstod() does not skip the wide space character.");
      DBUG_PRINTF ("test1", "Return Value: %lf", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the string starting at nptr has two or more wide characters which shows
   decimal point, verify the characters behind the 2nd wide character (include
   the 2nd one) which shows decimal point is not target of translation. */
private void
test2()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the string starting at nptr has two or more wide "
	       "characters which shows decimal point, verify the characters "
	       "behind the 2nd wide character (include the 2nd one) which "
	       "shows decimal point is not target of translation.";

  wchar_t str[] = L"4*4*4";

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = wcstod (str, NULL)) == 4.4)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("wcstod() does not handle decimal point character.");
      DBUG_PRINTF ("test2", "Return Value: %lf", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the numerical wide character string have a floating suffix, verify this
   function does not interpreted as numerical symbol. */
private void
test3()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the numerical wide character string have a floating "
	       "suffix, verify this function does not interpreted as "
	       "numerical symbol.";

  wchar_t str[] = L"4F";
  wchar_t *endptr;

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstod (str, &endptr);
  DBUG_PRINTF ("test3", "endptr: %lc", *endptr);
  
  if (*endptr == L'F')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not ignore a floating suffix.");
      DBUG_PRINTF ("test3", "Return Value: %lf", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the wide character string has a possibility to convert, check this
   function returns the converted value and endptr points the wide
   character which is next to the last numerical wide character. */
private void
test4()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string has a possibility to convert, "
	       "check this function returns the converted value and endptr "
	       "points the wide character which is next to the last "
	       "numerical wide character.";

  wchar_t str[] = L"4*23abcd";
  wchar_t *endptr;

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstod (str, &endptr);
  
  if (ret == 4.23)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test4", "Return Value: %lf", ret);
    }

  if (*endptr == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not set endptr correctry.");
      DBUG_PRINTF ("test4", "endptr: %lc", *endptr);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the wide character string has no possibility to convert, check this
   function returns 0 and endptr points the pointer which points the start
   of given string. */
private void
test5()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string has no possibility to convert, "
	       "check this function returns 0 and endptr points the pointer "
	       "which points the start of given string.";

  wchar_t str[] = L"abcd";
  wchar_t *endptr;

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstod (str, &endptr);
  
  if (ret == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test5", "Return Value: %lf", ret);
    }

  if (*endptr == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not set endptr correctry.");
      DBUG_PRINTF ("test5", "endptr: %lc", *endptr);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the numerical wide character string is bigger than the maximum value
   which handle by double type, this function returns the maximum value and
   set errno ERANGE. */
private void
test6()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the numerical wide character string is bigger than the "
	       "maximum value which handle by double type, this function "
	       "returns the maximum value and set errno ERANGE.";

  wchar_t str[STR_LENGTH];
  wchar_t *endptr;

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  str[1] = L'E';  str[2] = L'+';
  ret = wcstod (str, &endptr);
  
  if (ret == HUGE_VAL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test6", "Return Value: %lf", ret);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }


  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the numerical wide character string is smaller than the minimum value
   which handle by double type, this function returns the minimum value and
   set errno ERANGE. */
private void
test7()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the numerical wide character string is smaller than the "
	       "minimum value which handle by double type, this function "
	       "returns the minimum value and set errno ERANGE.";

  wchar_t str[STR_LENGTH];
  wchar_t *endptr;

  DBUG_ENTER ("test7");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  str[0] = L'-', str[2] = L'E';  str[3] = L'+';
  ret = wcstod (str, &endptr);
  
  if (ret == (-1) * HUGE_VAL)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test7", "Return Value: %lf", ret);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }


  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If underflow is occurred with converting, verify this function returns 0
   and set errno ERANGE. */
private void
test8()
{
  double ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If underflow is occurred with converting, verify this "
	       "function returns 0 and set errno ERANGE.";
  wchar_t str[STR_LENGTH];
  wchar_t *endptr;

  DBUG_ENTER ("test8");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  str[1] = L'E';  str[2] = L'-';
  ret = wcstod (str, &endptr);
  
  if (ret == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test8", "Return Value: %lf", ret);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }


  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}
