/*
 * LI18NUX2K.L1/base/wcstoul/wcstoul.c
 * 
 * Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
 * (X/Open)
 * trading as The Open Group
 * Copyright (c) 2001, 2003 International Business Machines Corp.
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

#include <std.h>
#include <dbug.h>
#include <report.h>
#include <tet_api.h>
#include <sysdep.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include <wchar.h>
#include <wctype.h>
#include <limits.h>
#include <errno.h>

#define STR_LENGTH 32

private char *test_locale = "LTP_1.UTF-8";

private void test1();
private void test2();
private void test3();
private void test4();
private void test5();
private void test6();
private void test7();
private void test8();
private void test9();
private void test10();

#define NO_TESTS 10
int	no_tests = NO_TESTS;
public 	void  	(*setfunc)()     = NULL;
public 	void  	(*clnfunc)()     = NULL;
public	struct tet_testlist tet_testlist[] = {
	test1,   1,
	test2,   2,
	test3,   3,
	test4,   4,
	test5,   5,
	test6,   6,
	test7,   7,
	test8,   8,
	test9,   9,
	test10, 10,
	NULL, 0
};


/* Check this function skips whitespace characters which is at beginning of
   the string starting at nptr. The whitespace character is the one whose
   results applied with iswspace() is true. */
private void
test1()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "Check this function skips whitespace characters which is at "
	       "beginning of the string starting at nptr. The whitespace "
	       "character is the one whose results applied with iswspace() "
	       "is true.";

  wchar_t str[] = L"    4";

  DBUG_ENTER ("test1");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  if ((ret = wcstoul (str, NULL, 0)) == 4)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("wcstoul() does not skip the wide space character.");
      DBUG_PRINTF ("test1", "Return Value: %ld", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If the numeral wide character string have a integer suffix, verify this
   function does not interpreted as numerical symbol. */
private void
test2()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the numeral wide character string have a integer suffix, "
	       "verify this function does not interpreted as numerical "
	       "symbol.";

  wchar_t str[] = L"4L";
  wchar_t *endptr;

  DBUG_ENTER ("test2");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 0);
  DBUG_PRINTF ("test2", "endptr: %lc", *endptr);
  
  if (*endptr == L'L')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not ignore a floating suffix.");
      DBUG_PRINTF ("test2", "Return Value: %ld", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If base is bigger than 9, verify some alphabets are interpreted as
   numerical character. (e.g. If base 16, 'a', 'b', 'c', 'd', 'e', 'f' are
   shows respectively '10', '11', '12', '13', '14', '15'.) */
private void
test3()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If base is bigger than 9, verify some alphabets are "
	       "interpreted as numerical character. (e.g. If base 16, 'a', "
	       "'b', 'c', 'd', 'e', 'f' are shows respectively '10', '11', "
	       "'12', '13', '14', '15'.)";

  wchar_t str[] = L"zz";
  wchar_t *endptr;

  DBUG_ENTER ("test3");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 36);
  
  if (ret == 35 * 36 + 35)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not recognized the base value.");
      DBUG_PRINTF ("test3", "Return Value: %ld", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If base is 16, verify the numerical string may have '0x' or '0X'. */
private void
test4()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If base is 16, verify the numerical string may have '0x' or "
	       "'0X'.";

  wchar_t str1[] = L"0xff", str2[] = L"0XFF";
  wchar_t *endptr;

  DBUG_ENTER ("test4");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str1, &endptr, 16);
  
  if (ret == 255)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not ignore \"0x\"");
      DBUG_PRINTF ("test4", "Return Value: %ld", ret);
    }

  ret = wcstoul (str2, &endptr, 16);
  
  if (ret == 255)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not ignore \"0x\"");
      DBUG_PRINTF ("test4", "Return Value: %ld", ret);
    }



  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If base is 0 and numerical string starts with '0', verify the string is
   taken as octal number. */
private void
test5()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If base is 0 and numerical string starts with '0', verify "
	       "the string is taken as octal number.";

  wchar_t str[] = L"077";
  wchar_t *endptr;

  DBUG_ENTER ("test5");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 0);
  
  if (ret == 7 * 8 + 7)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test5", "Return Value: %ld", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If base is 0 and numerical string starts without '0', verify the string is
   taken as decimal number. */
private void
test6()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If base is 0 and numerical string starts without '0', verify "
	       "the string is taken as decimal number.";

  wchar_t str[] = L"77";
  wchar_t *endptr;

  DBUG_ENTER ("test6");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 0);
  
  if (ret == 77)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test6", "Return Value: %ld", ret);
    }

  PATH_XS_RPT(1);
  DBUG_VOID_RETURN;
}


/* If base is 0 and numerical string starts with '0x' or '0X', verify the
   string is taken as hexaoctal number. */
private void
test7()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If base is 0 and numerical string starts with '0x' or '0X', "
	       "verify the string is taken as hexaoctal number.";

  wchar_t str1[] = L"0xff", str2[] = L"0XFF";
  wchar_t *endptr;

  DBUG_ENTER ("test7");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str1, &endptr, 0);
  
  if (ret == 255)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test7", "Return Value: %ld", ret);
    }

  ret = wcstoul (str2, &endptr, 0);
  
  if (ret == 255)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test7", "Return Value: %ld", ret);
    }



  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the wide character string has a possibility to convert, check this
   function returns the converted value and endptr points the wide
   character which is next to the last numerical wide character. */
private void
test8()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string has a possibility to convert, "
	       "check this function returns the converted value and endptr "
	       "points the wide character which is next to the last "
	       "numerical wide character.";

  wchar_t str[] = L"1234abcd";
  wchar_t *endptr;

  DBUG_ENTER ("test8");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 0);
  
  if (ret == 1234)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test8", "Return Value: %ld", ret);
    }

  ret = wcstoul (str, &endptr, 0);

  if (*endptr == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not set endptr correctry.");
      DBUG_PRINTF ("test8", "endptr: %lc", *endptr);
    }


  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the wide character string has no possibility to convert, check this
   function returns 0 and endptr points the pointer which points the start
   of given string. */
  private void
test9()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the wide character string has no possibility to convert, "
	       "check this function returns 0 and endptr points the pointer "
	       "which points the start of given string.";

  wchar_t str[] = L"abcd";
  wchar_t *endptr;

  DBUG_ENTER ("test9");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  ret = wcstoul (str, &endptr, 0);

  if (ret == 0)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test5", "Return Value: %ld", ret);
    }


  if (*endptr == L'a')
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("This function does not set endptr correctry.");
      DBUG_PRINTF ("test9", "endptr: %lc", *endptr);
    }

  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}


/* If the numerical wide character string is bigger than ULONG_MAX, verify this
   function returns ULONG_MAX and set errno ERANGE. */
private void
test10()
{
  unsigned long ret;
  char message[64] = "Test Locale is ";
  int  pathok = 0;
  char *desc = "If the numerical wide character string is bigger than "
	       "ULONG_MAX, verify this function returns ULONG_MAX and set "
	       "errno ERANGE.";

  wchar_t str[STR_LENGTH];
  wchar_t *endptr;

  DBUG_ENTER ("test10");
  tet_infoline (desc);

  if (setlocale(LC_ALL, test_locale) == NULL)
    {
      xx_rpt (FAILURE);
      in_rpt ("Cannot use test locale.");
      DBUG_VOID_RETURN;
    }

  strcat (message, test_locale);
  tet_infoline (message);

  globok = 1;

  wmemset (str, L'9', STR_LENGTH);
  ret = wcstoul (str, &endptr, 0);

  if (ret == ULONG_MAX)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The return value is not correct.");
      DBUG_PRINTF ("test10", "Return Value: %ld", ret);
    }

  if (errno == ERANGE)
    PATH_TRACE;
  else
    {
      xx_rpt (FAILURE);
      in_rpt ("The errno is not correct.");
    }


  PATH_XS_RPT(2);
  DBUG_VOID_RETURN;
}
