#/bin/sh
#
# LI18NUX2K.L1/gui/FontServer/FontServer.sh
#
# Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
# (X/Open)
# trading as The Open Group
# Copyright (c) 2001, 2003 International Business Machines Corp.
# All rights reserved except as granted by this License.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the "Artistic License" which comes with this
# Kit, with the following modification:
# a) "executable(s)" should be interpreted to include
# "test case(s)"
# b) if you wish to make changes as defined in clause 2 and 3, and
# distribute a modified version of this package, then
# clauses 3c and 4c are required
# c) Clause 7 is rephrased as follows: "Subroutines supplied by you
# and linked into this Package shall not be considered part of this
# Package".
#
#
# X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
# LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# You should have received a copy of the Artistic License with this
# Kit, in the file named "Artistic".  If not, we'll be glad to provide one.

# FontServer.sh : test case for checking 
#        if it has the capability which can display an outline font thru X Server.

tet_startup=""
tet_cleanup=""
iclist="ic1"
ic1="tp1"

DummyFontLong="-dummyoutline-gothic-medium-r-normal--0-[30 10.92 0 30]-0-0-p-0-iso10646-1"

DummyFontShort="dummyoutline"
ConfigFile=./config
ConfigOrig=./config.org

InitPort=9999
PortDef=/etc/services

TestOK=0

tp1()
{
	tet_infoline "Check if X Server has the capability which can display an outline font."

	xset q | fgrep "${LI18NUX_FONT_DIR}" > /dev/null 2>&1
	if [ $? != 0 ] ; then
		xset fp+ ${LI18NUX_FONT_DIR} > /dev/null 2>&1
		if [ $? != 0 ] ; then
		
			tet_infoline ">> The path of a dummy outline font was NOT added to the font path."
		
			########################################
			# xfs 
			pnum=`ps aux | grep "xfs -port ${LI18NUX_FONT_PORT}"| awk '{printf ("%d ",$2)}'`
			if [ "$pnum" = "" ] ; then
	
	
				CDIR=`echo ${LI18NUX_FONT_DIR} | sed 's/\//%/g'`
				sed "s/CDIR/$CDIR/g" $ConfigOrig | sed 's/%/\//g' > $ConfigFile
	
				########################################
				# start xfs server
				cmd="xfs -port ${LI18NUX_FONT_PORT} -config $ConfigFile -daemon"
				echo "++ enter the root password to start font server"
				su root -c "$cmd"
				ps aux | grep xfs | fgrep -v grep > /dev/null 2>&1
				if [ $? = 1 ] ; then
					tet_infoline ">> xfs has not been started, so aborted."
					exit 1
				fi
			fi
		
			########################################
			# xset +fp
			xset +fp unix/:${LI18NUX_FONT_PORT} > /dev/null 2>&1
			if [ $? != 0 ] ; then
				tet_infoline "fatal error: xset fp+ unix/:${LI18NUX_FONT_PORT}"
				exit 1
			fi
		
			########################################
			# xlsfonts check
			xlsfonts | grep $DummyFontShort > /dev/null 2>&1
			if [ $? = 0 ] ; then
				# display the font
				# xfd -fn "$DummyFontLong" 
				./dispout
				if [ $? = 0 ] ; then
					TestOK=1
				fi
			fi
		
			########################################
			# clean up
			# pnum=`ps aux | grep "xfs -port ${LI18NUX_FONT_PORT}"| awk '{printf ("%d ",$2)}'`
			# if [ "$pnum" != "" ] ; then
			# 	cmd="kill -9 $pnum"
			# 	echo "++ enter the root password to clean up font server process"
			# 	su root -c "$cmd"
			# fi
			xset -fp unix/:${LI18NUX_FONT_PORT}
		
		else
			tet_infoline ">> The path of a dummy outline font was added to the font path."
		
			########################################
			# xlsfonts check
			xlsfonts | grep $DummyFontShort > /dev/null 2>&1
			if [ $? = 0 ] ; then
				# display the font
				# xfd -fn "$DummyFontLong"
				./dispout
				if [ $? = 0 ] ; then
					TestOK=1
				fi
			fi
		
			########################################
			# clean up
			xset fp- ${LI18NUX_FONT_DIR} > /dev/null 2>&1
		fi
	else
		tet_infoline ">> The path of a dummy outline font was added to the font path."
	
		########################################
		# xlsfonts check
		xlsfonts | grep $DummyFontShort > /dev/null 2>&1
		if [ $? = 0 ] ; then
			# display the font
			# xfd -fn "$DummyFontLong"
			./dispout
			if [ $? = 0 ] ; then
				TestOK=1
			fi
		fi
	
		########################################
		# clean up
		xset fp- ${LI18NUX_FONT_DIR} > /dev/null 2>&1
	fi
	
	if [ $TestOK = 1 ] ; then
		tet_infoline "X Server is supporting the outline font."
		tet_result PASS # success
	else
		tet_infoline "X Server is NOT supporting the outline font."
		tet_result FAIL # fail
	fi
}


# source common shell functions
. $TET_EXECUTE/LIB/i18nfuncs.sh
. $TET_EXECUTE/LIB/shfuncs.sh
. $TET_EXECUTE/LIB/lsbfuncs.sh

# execute shell test case manager - must be last line
. $TET_ROOT/lib/xpg3sh/tcm.sh
